local VERSION = tonumber(("$Revision: 44639 $"):match("%d+"))

local Parrot = Parrot
local Parrot_Triggers = Parrot:NewModule("Triggers", "AceEvent-2.0", "Parser-3.0")
if Parrot.revision < VERSION then
	Parrot.version = "r" .. VERSION
	Parrot.revision = VERSION
	Parrot.date = ("$Date: 2007-07-23 14:11:24 -0400 (Mon, 23 Jul 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

local L = Parrot:L("Parrot_Triggers")

local BS = AceLibrary("Babble-Spell-2.2")

local new, del, newSet, newHash = Parrot.new, Parrot.del, Parrot.newSet, Parrot.newHash

local _,playerClass = UnitClass("player")

local SharedMedia = AceLibrary("SharedMedia-1.0")

Parrot_Triggers.db = Parrot:AcquireDBNamespace("Triggers")
Parrot:RegisterDefaults("Triggers", 'profile', {})

local effectiveRegistry = {}
local function rebuildEffectiveRegistry()
	for i = 1, #effectiveRegistry do
		effectiveRegistry[i] = nil
	end
	for _,v in ipairs(Parrot_Triggers.db.profile.triggers) do
		if not v.disabled then
			local classes = newSet((';'):split(v.class))
			if classes[playerClass] then
				effectiveRegistry[#effectiveRegistry+1] = v
			end
			classes = del(classes)
		end
	end
end

local cooldowns = {}
local currentTrigger
local Parrot_TriggerConditions
local Parrot_Display
local Parrot_ScrollAreas
local Parrot_CombatEvents
function Parrot_Triggers:OnEnable(first)
	if not self.db.profile.triggers then
		self.db.profile.triggers = {
			{
				name = L["%s!"]:format(BS["Backlash"]),
				class = "WARLOCK",
				conditions = {
					["Self buff gain"] = BS["Backlash"],
				},
				sticky = true,
				color = "ff00ff",
			},
			{
				name = L["%s!"]:format(BS["Blackout"]),
				class = "PRIEST",
				conditions = {
					["Target debuff gain"] = BS["Blackout"],
				},
				sticky = true,
				color = "ff00ff",
			},
			{
				name = L["%s!"]:format(BS["Clearcasting"]),
				class = "MAGE;PRIEST;SHAMAN",
				conditions = {
					["Self buff gain"] = BS["Clearcasting"],
				},
				sticky = true,
				color = "ffff00",
			},
			{
				name = L["%s!"]:format(BS["Counterattack"]),
				class = "HUNTER",
				conditions = {
					["Incoming parry"] = true,
				},
				secondaryConditions = {
					["Spell ready"] = BS["Counterattack"],
				},
				sticky = true,
				color = "ffff00",
			},
			{
				name = L["%s!"]:format(BS["Execute"]),
				class = "WARRIOR",
				conditions = {
					["Enemy target health percent"] = 0.2,
				},
				secondaryConditions = {
					["Spell ready"] = BS["Execute"],
				},
				sticky = true,
				color = "ffff00",
			},
			{
				name = L["%s!"]:format(BS["Frostbite"]),
				class = "MAGE",
				conditions = {
					["Target debuff gain"] = BS["Frostbite"],
				},
				sticky = true,
				color = "0000ff",
			},
			{
				name = L["%s!"]:format(BS["Hammer of Wrath"]),
				class = "PALADIN",
				conditions = {
					["Enemy target health percent"] = 0.2,
				},
				secondaryConditions = {
					["Spell ready"] = BS["Hammer of Wrath"],
				},
				sticky = true,
				color = "ffff00",
			},
			{
				name = L["%s!"]:format(BS["Impact"]),
				class = "MAGE",
				conditions = {
					["Target debuff gain"] = BS["Impact"],
				},
				sticky = true,
				color = "ff0000",
			},
			{
				name = L["%s!"]:format(BS["Kill Command"]),
				class = "HUNTER",
				conditions = {
					["Outgoing crit"] = true,
				},
				secondaryConditions = {
					["Spell ready"] = BS["Kill Command"],
				},
				sticky = true,
				color = "ff0000",
				disabled = true,
			},
			{
				name = L["Low Health!"],
				class = "DRUID;HUNTER;MAGE;PALADIN;PRIEST;ROGUE;SHAMAN;WARLOCK;WARRIOR",
				conditions = {
					["Self health percent"] = 0.4,
				},
				secondaryConditions = {
					["Trigger cooldown"] = 3,
				},
				sticky = true,
				color = "ff7f7f",
			},
			{
				name = L["Low Mana!"],
				class = "DRUID;HUNTER;MAGE;PALADIN;PRIEST;SHAMAN;WARLOCK",
				conditions = {
					["Self mana percent"] = 0.35,
				},
				secondaryConditions = {
					["Trigger cooldown"] = 3,
				},
				sticky = true,
				color = "7f7fff",
			},
			{
				name = L["Low Pet Health!"],
				class = "HUNTER;MAGE;WARLOCK",
				conditions = {
					["Pet health percent"] = 0.4,
				},
				secondaryConditions = {
					["Trigger cooldown"] = 3,
				},
				color = "ff7f7f",
			},
			{
				name = L["%s!"]:format(BS["Mongoose Bite"]),
				class = "HUNTER",
				conditions = {
					["Incoming dodge"] = true,
				},
				secondaryConditions = {
					["Spell ready"] = BS["Mongoose Bite"],
				},
				sticky = true,
				color = "ffff00",
			},
			{
				name = L["%s!"]:format(BS["Nightfall"]),
				class = "WARLOCK",
				conditions = {
					["Self buff gain"] = BS["Shadow Trance"],
				},
				sticky = true,
				color = "7f007f",
			},
			{
				name = L["Free %s!"]:format(BS["Smite"]),
				class = "PRIEST",
				conditions = {
					["Self buff gain"] = BS["Surge of Light"],
				},
				sticky = true,
				disabled = true,
				color = "ff0000",
			},			
			{
				name = L["%s!"]:format(BS["Overpower"]),
				class = "WARRIOR",
				conditions = {
					["Outgoing dodge"] = true,
				},
				secondaryConditions = {
					["Spell ready"] = BS["Overpower"],
				},
				sticky = true,
				color = "7f007f",
			},
			{
				name = L["%s!"]:format(BS["Rampage"]),
				class = "WARRIOR",
				conditions = {
					["Outgoing crit"] = true,
				},
				secondaryConditions = {
					["Spell ready"] = BS["Rampage"],
					["Buff inactive"] = BS["Rampage"],
					["Minimum power amount"] = 20,
				},
				sticky = true,
				color = "ff0000",
			},
			{
				name = L["%s!"]:format(BS["Revenge"]),
				class = "WARRIOR",
				conditions = {
					["Incoming block"] = true,
					["Incoming dodge"] = true,
					["Incoming parry"] = true,
				},
				secondaryConditions = {
					["Spell ready"] = BS["Revenge"],
					["Warrior stance"] = "Defensive Stance",
				},
				sticky = true,
				color = "ffff00",
				disabled = true,
			},
			{
				name = L["%s!"]:format(BS["Riposte"]),
				class = "ROGUE",
				conditions = {
					["Incoming parry"] = true,
				},
				secondaryConditions = {
					["Spell ready"] = BS["Riposte"],
				},
				sticky = true,
				color = "ffff00",
			},
		}
	end
	Parrot_Display = Parrot:GetModule("Display")
	Parrot_ScrollAreas = Parrot:GetModule("ScrollAreas")
	Parrot_TriggerConditions = Parrot:GetModule("TriggerConditions")
	Parrot_CombatEvents = Parrot:GetModule("CombatEvents")
	if first then
		Parrot_TriggerConditions:RegisterSecondaryTriggerCondition {
			name = "Trigger cooldown",
			localName = L["Trigger cooldown"],
			defaultParam = 3,
			param = {
				type = "range",
				min = 0,
				max = 60,
				step = 0.1,
				bigStep = 1,
			},
			check = function(param)
				if not cooldowns[currentTrigger] then
					return true
				end
				local now = GetTime()
				return now - cooldowns[currentTrigger] > param
			end,
		}
		
		for _,data in ipairs(self.db.profile.triggers) do
			local t = new()
			for k,v in pairs(data.conditions) do
				t[k] = v
				data.conditions[k] = nil
			end
			local validate = Parrot_TriggerConditions:GetPrimaryConditionValidate()
			for k,v in pairs(t) do
				if validate[k] then
					data.conditions[k] = v
					t[k] = nil
				else
					local k_lower = k:lower()
					for l,u in pairs(validate) do
						if l:lower() == k_lower then
							data.conditions[l] = v
							t[k] = nil
						end
					end
					if t[k] then
						data.conditions[k] = v
						t[k] = nil
					end
				end
			end
			t = del(t)
			if data.secondaryConditions then
				local t = new()
				for k,v in pairs(data.secondaryConditions) do
					t[k] = v
					data.secondaryConditions[k] = nil
				end
				local validate = Parrot_TriggerConditions:GetSecondaryConditionValidate()
				for k,v in pairs(t) do
					if validate[k] then
						data.secondaryConditions[k] = v
						t[k] = nil
					else
						local k_lower = k:lower()
						for l,u in pairs(validate) do
							if l:lower() == k_lower then
								data.secondaryConditions[l] = v
								t[k] = nil
							end
						end
						if t[k] then
							data.secondaryConditions[k] = v
							t[k] = nil
						end
					end
				end
				t = del(t)
			end
		end
	end
	
	rebuildEffectiveRegistry()
end

local numberedConditions = {}

local function hexColorToTuple(color)
	local num = tonumber(color, 16)
	return math.floor(num / 256^2)/255, math.floor((num / 256)%256)/255, (num%256)/255
end

function Parrot_Triggers:OnTriggerCondition(name, arg, uid)
	for _,v in ipairs(effectiveRegistry) do
		local conditions = v.conditions
		if conditions then
			local param = conditions[name]
			if param then
				local good = false
				if param == true then
				 	good = true
				elseif type(arg) == "string" then
					good = param == arg
				elseif type(arg) == "number" then
					if not numberedConditions[v] then
						numberedConditions[v] = new()
					end
					good = arg <= param and (not numberedConditions[v][name] or numberedConditions[v][name] > param)
					numberedConditions[v][name] = arg
				end
				if good then
					local secondaryConditions = v.secondaryConditions
					if secondaryConditions then
						currentTrigger = v.name
						for k, v in pairs(secondaryConditions) do
							if not Parrot_TriggerConditions:DoesSecondaryTriggerConditionPass(k, v) then
								good = false
								break
							end
						end
					end
					if good and Parrot_TriggerConditions:DoesSecondaryTriggerConditionPass("Trigger cooldown", 0.1) then
						cooldowns[v.name] = GetTime()
						local r, g, b = hexColorToTuple(v.color or 'ffffff')
						Parrot_Display:ShowMessage(v.name, v.scrollArea or "Notification", v.sticky, r, g, b, v.font, v.fontSize, v.outline)
						if v.sound then
							PlaySoundFile(SharedMedia:Fetch('sound', v.sound))
						end
						if uid then
							Parrot_CombatEvents:CancelEventsWithUID(uid)
						end
					end
				end
			end
		end
	end
end

function Parrot_Triggers:OnOptionsCreate()
	local makeOption
	Parrot.options.args.triggers = {
		type = 'group',
		name = L["Triggers"],
		desc = L["Triggers"],
		args = {
			{
				type = 'execute',
				name = L["New trigger"],
				desc = L["Create a new trigger"],
				func = function()
					local t = {
						name = L["New trigger"],
						class = "DRUID;HUNTER;MAGE;PALADIN;PRIEST;ROGUE;SHAMAN;WARLOCK;WARRIOR",
						conditions = {},
					}
					local registry = self.db.profile.triggers
					registry[#registry+1] = t
					makeOption(t)
					rebuildEffectiveRegistry()
				end,
				disabled = function()
					for _,v in ipairs(self.db.profile.triggers) do
						if v.name == L["New trigger"] then
							return true
						end
					end
					return false
				end
			}
		}
	}
	
	local function getFontFace(t)
		local font = t.font
		if font == nil then
			return L["Inherit"]
		else
			return font
		end
	end
	local function setFontFace(t, value)
		if value == L["Inherit"] then
			value = nil
		end
		t.font = value
	end
	local function getFontSize(t)
		return t.fontSize
	end
	local function setFontSize(t, value)
		t.fontSize = value
	end
	local function getFontSizeInherit(t)
		return t.fontSize == nil
	end
	local function setFontSizeInherit(t, value)
		if value then
			t.fontSize = nil
		else
			t.fontSize = 18
		end
	end
	local function getFontOutline(t)
		local outline = t.fontOutline
		if outline == nil then
			return L["Inherit"]
		else
			return outline
		end
	end
	local function setFontOutline(t, value)
		if value == L["Inherit"] then
			value = nil
		end
		t.fontOutline = value
	end
	local fontOutlineValidate = {
		NONE = L["None"],
		OUTLINE = L["Thin"],
		THICKOUTLINE = L["Thick"],
		[L["Inherit"]] = L["Inherit"],
	}
	local function getEnabled(t)
		return not t.disabled
	end
	local function setEnabled(t, value)
		t.disabled = not value or nil
		rebuildEffectiveRegistry()
	end
	local function getScrollArea(t)
		return t.scrollArea or "Notification"
	end
	local function setScrollArea(t, value)
		if value == "Notification" then
			value = nil
		end
		t.scrollArea = value
	end
	local function remove(t)
		Parrot.options.args.triggers.args[tostring(t)] = nil
		for i,v in ipairs(self.db.profile.triggers) do
			if v == t then
				table.remove(self.db.profile.triggers, i)
				break
			end
		end
		rebuildEffectiveRegistry()
	end
	local function getSticky(t)
		return t.sticky
	end
	local function setSticky(t, value)
		t.sticky = value or nil
	end
	local function getName(t)
		return t.name
	end
	local function setName(t, value)
		t.name = value
		local opt = Parrot.options.args.triggers.args[tostring(t)]
		opt.name = value
		opt.desc = value
		opt.order = value == L["New trigger"] and -110 or -100
	end
	
	local function tupleToHexColor(r, g, b)
		return ("%02x%02x%02x"):format(r * 255, g * 255, b * 255)
	end
	
	local function getColor(t)
		return hexColorToTuple(t.color or "ffffff")
	end
	local function setColor(t, r, g, b)
		local color = tupleToHexColor(r, g, b)
		if color == "ffffff" then
			color = nil
		end
		t.color = color
	end
	
	local function getClass(t, class)
		local tmp = newSet((";"):split(t.class))
		local value = tmp[class]
		tmp = del(tmp)
		return value
	end
	
	local function setClass(t, class, value)
		local tmp = newSet((";"):split(t.class))
		tmp[class] = value or nil
		local tmp2 = new()
		for k in pairs(tmp) do
			tmp2[#tmp2+1] = k
		end
		tmp = del(tmp)
		t.class = table.concat(tmp2, ";")
		tmp2 = del(tmp2)
		if class == playerClass then
			rebuildEffectiveRegistry()
		end
	end
	
	local function getSound(t)
		return t.sound or "None"
	end
	
	local function setSound(t, value)
		PlaySoundFile(SharedMedia:Fetch('sound', value))
		if value == "None" then
			value = nil
		end
		t.sound = value
	end
	
	local function test(t)
		local r, g, b = hexColorToTuple(t.color or 'ffffff')
		Parrot_Display:ShowMessage(t.name, t.scrollArea or "Notification", t.sticky, r, g, b, t.font, t.fontSize, t.outline)
		if t.sound then
			PlaySoundFile(SharedMedia:Fetch('sound', t.sound))
		end
	end
	
	local classValidate = {
		DRUID = L["Druid"],
		ROGUE = L["Rogue"],
		SHAMAN = L["Shaman"],
		PALADIN = L["Paladin"],
		MAGE = L["Mage"],
		WARLOCK = L["Warlock"],
		PRIEST = L["Priest"],
		WARRIOR = L["Warrior"],
		HUNTER = L["Hunter"],
	}
	
	local function addPrimaryCondition(t, name, localName)
		local opt = Parrot.options.args.triggers.args[tostring(t)].args.primary
		local param, default = Parrot_TriggerConditions:GetPrimaryConditionParamDetails(name)
		if not param then
			opt.args[name] = newHash(
				'type', 'execute',
				'name', localName,
				'desc', localName,
				'func', function() end,
				'order', -100
			)
			return true
		else
			local tmp = newHash(
				'name', localName,
				'desc', localName,
				'get', function()
					return t.conditions[name]
				end,
				'set', function(value)
					t.conditions[name] = value
				end,
				'order', -100
			)
			for k, v in pairs(param) do
				tmp[k] = v
			end
			opt.args[name] = tmp
			if default then
				return default
			end
			if type(param.min) == "number" and type(param.max) == "number" then
				return (param.max + param.min) / 2
			end
			return false
		end
	end
	local function newPrimaryCondition(t, name)
		local opt = Parrot.options.args.triggers.args[tostring(t)].args.primary
		local localName = opt.args[1].validate[name]
		opt.args[1].validate[name] = nil
		opt.args[2].validate[name] = localName
		t.conditions[name] = addPrimaryCondition(t, name, localName)
	end
	local function removePrimaryCondition(t, name)
		local opt = Parrot.options.args.triggers.args[tostring(t)].args.primary
		opt.args[1].validate[name], opt.args[2].validate[name] = opt.args[2].validate[name], opt.args[1].validate[name]
		opt.args[name] = del(opt.args[name])
		t.conditions[name] = nil
	end
	local function hasNoPrimaryConditions(t)
		return next(t.conditions) == nil
	end
	local function hasAllPrimaryConditions(t)
		for k,v in pairs(Parrot_TriggerConditions:GetPrimaryConditionValidate()) do
			if t.conditions[k] == nil then
				return false
			end
		end
		return true
	end
	
	local function addSecondaryCondition(t, name, localName)
		local opt = Parrot.options.args.triggers.args[tostring(t)].args.secondary
		local param, default = Parrot_TriggerConditions:GetSecondaryConditionParamDetails(name)
		if not param then
			opt.args[name] = newHash(
				'type', 'execute',
				'name', localName,
				'desc', localName,
				'func', function() end,
				'order', -100
			)
			return true
		else
			local tmp = newHash(
				'name', localName,
				'desc', localName,
				'get', function()
					return t.secondaryConditions[name]
				end,
				'set', function(value)
					t.secondaryConditions[name] = value
				end,
				'order', -100
			)
			for k, v in pairs(param) do
				tmp[k] = v
			end
			opt.args[name] = tmp
			if default then
				return default
			end
			if type(param.min) == "number" and type(param.max) == "number" then
				return (param.max + param.min) / 2
			end
			return false
		end
	end
	local function newSecondaryCondition(t, name)
		local opt = Parrot.options.args.triggers.args[tostring(t)].args.secondary
		local localName = opt.args[1].validate[name]
		opt.args[1].validate[name] = nil
		opt.args[2].validate[name] = localName
		if not t.secondaryConditions then
			t.secondaryConditions = new()
		end
		t.secondaryConditions[name] = addSecondaryCondition(t, name, localName)
	end
	local function removeSecondaryCondition(t, name)
		local opt = Parrot.options.args.triggers.args[tostring(t)].args.secondary
		opt.args[1].validate[name], opt.args[2].validate[name] = opt.args[2].validate[name], opt.args[1].validate[name]
		opt.args[name] = del(opt.args[name])
		t.secondaryConditions[name] = nil
		if next(t.secondaryConditions) == nil then
			t.secondaryConditions = del(t.secondaryConditions)
		end
	end
	local function hasNoSecondaryConditions(t)
		return not t.secondaryConditions
	end
	local function hasAllSecondaryConditions(t)
		if not t.secondaryConditions then
			return false
		end
		for k,v in pairs(Parrot_TriggerConditions:GetSecondaryConditionValidate()) do
			if t.secondaryConditions[k] == nil then
				return false
			end
		end
		return true
	end
	
	function makeOption(t)
		local opt = {
			type = 'group',
			name = t.name,
			desc = t.name,
			order = t.name == L["New trigger"] and -110 or -100,
			args = {
				output = {
					type = 'text',
					name = L["Output"],
					desc = L["The text that is shown"],
					usage = '<Text to show>',
					get = getName,
					set = setName,
					passValue = t,
					order = 1,
				},
				enabled = {
					type = 'toggle',
					name = L["Enabled"],
					desc = L["Whether the trigger is enabled or not."],
					get = getEnabled,
					set = setEnabled,
					passValue = t,
					order = -1,
				},
				remove = {
					type = 'execute',
					name = L["Remove trigger"],
					desc = L["Remove this trigger completely."],
					func = remove,
					passValue = t,
					confirm = true,
					order = -2,
				},
				color = {
					name = L["Color"],
					desc = L["Color of the text for this trigger."],
					type = 'color',
					get = getColor,
					set = setColor,
					passValue = t,
				},
				sticky = {
					type = 'toggle',
					name = L["Sticky"],
					desc = L["Whether to show this trigger as a sticky."],
					get = getSticky,
					set = setSticky,
					passValue = t,
				},
				classes = {
					type = 'text',
					multiToggle = true,
					validate = classValidate,
					name = L["Classes"],
					desc = L["Classes affected by this trigger."],
					get = getClass,
					set = setClass,
					passValue = t,
				},
				scrollArea = {
					type = 'text',
					validate = Parrot_ScrollAreas:GetScrollAreasValidate(),
					name = L["Scroll area"],
					desc = L["Which scroll area to output to."],
					get = getScrollArea,
					set = setScrollArea,
					passValue = t,
				},
				sound = {
					type = 'text',
					validate = SharedMedia:List("sound"),
					name = L["Sound"],
					desc = L["What sound to play when the trigger is shown."],
					get = getSound,
					set = setSound,
					passValue = t,
				},
				test = {
					type = 'execute',
					name = L["Test"],
					desc = L["Test how the trigger will look and act."],
					func = test,
					passValue = t,
				},
				font = {
					type = 'group',
					name = L["Custom font"],
					desc = L["Custom font"],
					args = {
						fontface = {
							type = 'text',
							name = L["Font face"],
							desc = L["Font face"],
							validate = Parrot.inheritFontValidate,
							get = getFontFace,
							set = setFontFace,
							passValue = t,
							order = 1,
						},
						fontSizeInherit = {
							type = 'toggle',
							name = L["Inherit font size"],
							desc = L["Inherit font size"],
							get = getFontSizeInherit,
							set = setFontSizeInherit,
							passValue = t,
							order = 2,
						},
						fontSize = {
							type = 'range',
							name = L["Font size"],
							desc = L["Font size"],
							min = 12,
							max = 30,
							step = 1,
							get = getFontSize,
							set = setFontSize,
							disabled = getFontSizeInherit,
							passValue = t,
							order = 3,
						},
						fontOutline = {
							type = 'text',
							name = L["Font outline"],
							desc = L["Font outline"],
							get = getFontOutline,
							set = setFontOutline,
							validate = fontOutlineValidate,
							passValue = t,
							order = 4,
						},
					},
				},
				primary = {
					type = 'group',
					name = L["Primary conditions"],
					desc = L["When any of these conditions apply, the secondary conditions are checked."],
					args = {
						{
							type = 'text',
							name = L["New condition"],
							desc = L["Add a new primary condition"],
							validate = {},
							get = false,
							set = newPrimaryCondition,
							disabled = hasAllPrimaryConditions,
							passValue = t,
							order = 1,
						},
						{
							type = 'text',
							name = L["Remove condition"],
							desc = L["Remove a primary condition"],
							validate = {},
							get = false,
							set = removePrimaryCondition,
							disabled = hasNoPrimaryConditions,
							passValue = t,
							order = 2,
						}
					}
				},
				secondary = {
					type = 'group',
					name = L["Secondary conditions"],
					desc = L["When all of these conditions apply, the trigger will be shown."],
					args = {
						{
							type = 'text',
							name = L["New condition"],
							desc = L["Add a new secondary condition"],
							validate = {},
							get = false,
							set = newSecondaryCondition,
							disabled = hasAllSecondaryConditions,
							passValue = t,
							order = 1,
						},
						{
							type = 'text',
							name = L["Remove condition"],
							desc = L["Remove a secondary condition"],
							validate = {},
							get = false,
							set = removeSecondaryCondition,
							disabled = hasNoSecondaryConditions,
							passValue = t,
							order = 2,
						}
					}
				},
			}
		}
		Parrot.options.args.triggers.args[tostring(t)] = opt
		for k,v in pairs(Parrot_TriggerConditions:GetPrimaryConditionValidate()) do
			opt.args.primary.args[t.conditions[k] and 2 or 1].validate[k] = v
			if t.conditions[k] then
				addPrimaryCondition(t, k, v)
			end
		end
		for k,v in pairs(Parrot_TriggerConditions:GetSecondaryConditionValidate()) do
			opt.args.secondary.args[t.secondaryConditions and t.secondaryConditions[k] and 2 or 1].validate[k] = v
			if t.secondaryConditions and t.secondaryConditions[k] then
				addSecondaryCondition(t, k, v)
			end
		end
	end
	
	for _,t in ipairs(self.db.profile.triggers) do
		makeOption(t)
	end
end