local VERSION = tonumber(("$Revision: 44702 $"):match("%d+"))

local Parrot = Parrot
if Parrot.revision < VERSION then
	Parrot.version = "r" .. VERSION
	Parrot.revision = VERSION
	Parrot.date = ("$Date: 2007-07-24 01:02:48 -0400 (Tue, 24 Jul 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

local mod = Parrot:NewModule("CombatEvents_Data", "AceEvent-2.0", "Parser-3.0")

local BS = AceLibrary("Babble-Spell-2.2")
local Gratuity -- AceLibrary("Gratuity-2.0")

local L = Parrot:L("Parrot_CombatEvents_Data", {
	["Miss!"] = MISS .. "!",
	["Dodge!"] = DODGE .. "!",
	["Parry!"] = PARRY .. "!",
	["Block!"] = BLOCK .. "!",
	["Absorb!"] = ABSORB .. "!",
	["Immune!"] = IMMUNE .. "!",
	["Resist!"] = RESIST .. "!",
	["Reflect!"] = REFLECT .. "!",
	["Interrupt!"] = INTERRUPT .. "!",
	["Pet Miss!"] = PET .. " " .. MISS .. "!",
	["Pet Dodge!"] = PET .. " " .. DODGE .. "!",
	["Pet Parry!"] = PET .. " " .. PARRY .. "!",
	["Pet Block!"] = PET .. " " .. BLOCK .. "!",
	["Pet Absorb!"] = PET .. " " .. ABSORB .. "!",
	["Pet Immune!"] = PET .. " " .. IMMUNE .. "!",
	["Pet Resist!"] = PET .. " " .. RESIST .. "!",
	["Pet Reflect!"] = PET .. " " .. REFLECT .. "!",
	["Pet Interrupt!"] = PET .. " " .. INTERRUPT .. "!",
})

local new, del = Parrot.new, Parrot.del

local onEnableFuncs = {}
function mod:OnEnable()
	assert(AceLibrary("SpecialEvents-Aura-2.0")) -- force a load
	Gratuity = AceLibrary("Gratuity-2.0")
	
	for _,v in ipairs(onEnableFuncs) do
		v()
	end
end

------------------------------------------------------------------------------
-- Incoming events -----------------------------------------------------------
------------------------------------------------------------------------------
local coloredDamageAmount = function(info)
	local db = Parrot:GetModule("CombatEvents").db.profile.damageTypes
	if db.color then
		return "|cff" .. db[info.damageType] .. info.amount .. "|r"
	else
		return info.amount
	end
end

Parrot:RegisterFilterType("Incoming damage", L["Incoming damage"], 0)

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee damage",
	localName = L["Melee damage"],
	defaultTag = "([Name]) -[Amount]",
	parserEvent = {
		eventType = "Damage",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Amount = coloredDamageAmount,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
		Amount = L["The amount of damage done."],
	},
	color = "ff0000", -- red
	canCrit = true,
	filterType = { "Incoming damage", 'amount' },
}

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee misses",
	localName = L["Melee misses"],
	defaultTag = L["Miss!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Miss",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
	},
	color = "0000ff", -- blue
}

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee dodges",
	localName = L["Melee dodges"],
	defaultTag = L["Dodge!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Dodge",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
	},
	color = "0000ff", -- blue
}

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee parries",
	localName = L["Melee parries"],
	defaultTag = L["Parry!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Parry",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
	},
	color = "0000ff", -- blue
}

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee blocks",
	localName = L["Melee blocks"],
	defaultTag = L["Block!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Block",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
	},
	color = "0000ff", -- blue
}

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee absorbs",
	localName = L["Melee absorbs"],
	defaultTag = L["Absorb!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Absorb",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
	},
	color = "ffff00", -- yellow
}

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee immunes",
	localName = L["Melee immunes"],
	defaultTag = L["Immune!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Immune",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
	},
	color = "ffff00", -- yellow
}

local function retrieveAbilityName(info)
	return Parrot:GetModule("CombatEvents"):GetAbbreviatedSpell(info.abilityName)
end

local function retrieveIconFromAbilityName(info)
	local spell = info.abilityName
	if BS:HasReverseTranslation(spell) then
		return BS:GetSpellIcon(BS.reverse[spell])
	end
end

local function retrieveIconFrom1(info)
	local spell = info[1]
	if BS:HasReverseTranslation(spell) then
		return BS:GetSpellIcon(BS.reverse[spell])
	end
end

Parrot:RegisterThrottleType("Skill damage", L["Skill damage"], 0.1, true)

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Skill damage",
	localName = L["Skill damage"],
	defaultTag = "([Name]) -[Amount]",
	parserEvent = {
		eventType = "Damage",
		recipientID = "player",
		abilityName_not = false,
		isDoT = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Amount = coloredDamageAmount,
		Type = "damageTypeLocal",
		Skill = retrieveAbilityName,
		Count = "throttleCount",
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
		Amount = L["The amount of damage done."],
		Type = L["The type of damage done."],
		Skill = L["The spell or ability that the enemy attacked you with."],
		Count = L["The amount of spell casts to make up the throttled event."],
	},
	color = "ff0000", -- red
	canCrit = true,
	throttle = { "Skill damage", 'abilityName', 'throttleCount', sourceName = L["Multiple"] },
	filterType = { "Incoming damage", 'amount' },
}

Parrot:RegisterThrottleType("DoTs and HoTs", L["DoTs and HoTs"], 2)

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Skill DoTs",
	localName = L["Skill DoTs"],
	defaultTag = "([Name]) -[Amount]",
	parserEvent = {
		eventType = "Damage",
		recipientID = "player",
		abilityName_not = false,
		isDoT = true,
	},
	tagTranslations = {
		Name = "sourceName",
		Amount = coloredDamageAmount,
		Type = "damageTypeLocal",
		Skill = retrieveAbilityName,
		Count = "throttleCount",
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
		Amount = L["The amount of damage done."],
		Type = L["The type of damage done."],
		Skill = L["The spell or ability that the enemy attacked you with."],
		Count = L["The amount of spell casts to make up the throttled event."],
	},
	color = "ff0000", -- red
	throttle = { "DoTs and HoTs", 'abilityName', 'throttleCount', sourceName = L["Multiple"] },
}

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Ability misses",
	localName = L["Ability misses"],
	defaultTag = "([Skill]) " .. L["Miss!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Miss",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
		Skill = L["The spell or ability that the enemy attacked you with."],
	},
	color = "0000ff", -- blue
}

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Ability dodges",
	localName = L["Ability dodges"],
	defaultTag = "([Skill]) " .. L["Dodge!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Dodge",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
		Skill = L["The spell or ability that the enemy attacked you with."],
	},
	color = "0000ff", -- blue
}

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Ability parries",
	localName = L["Ability parries"],
	defaultTag = "([Skill]) " .. L["Parry!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Parry",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
		Skill = L["The spell or ability that the enemy attacked you with."],
	},
	color = "0000ff", -- blue
}

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Ability blocks",
	localName = L["Ability blocks"],
	defaultTag = "([Skill]) " .. L["Block!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Block",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
		Skill = L["The spell or ability that the enemy attacked you with."],
	},
	color = "0000ff", -- blue
}

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Spell resists",
	localName = L["Spell resists"],
	defaultTag = "([Skill]) " .. L["Resist!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Resist",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
		Skill = L["The spell or ability that the enemy attacked you with."],
	},
	color = "7f007f", -- purple
}

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Skill absorbs",
	localName = L["Skill absorbs"],
	defaultTag = "([Skill]) " .. L["Absorb!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Absorb",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
		Skill = L["The spell or ability that the enemy attacked you with."],
	},
	color = "ffff00", -- yellow
}

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Skill immunes",
	localName = L["Skill immunes"],
	defaultTag = "([Skill]) " .. L["Immune!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Immune",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
		Skill = L["The spell or ability that the enemy attacked you with."],
	},
	color = "ffff00", -- yellow
}

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Skill reflects",
	localName = L["Skill reflects"],
	defaultTag = "([Skill]) " .. L["Reflect!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Reflect",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
		Skill = L["The spell or ability that the enemy attacked you with."],
	},
	color = "7f007f", -- purple
	canCrit = true,
}

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Spell interrupts",
	localName = L["Skill interrupts"],
	defaultTag = "([Skill]) " .. L["Interrupt!"],
	parserEvent = {
		eventType = "Interrupt",
		recipientID = "player",
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy that attacked you."],
		Skill = L["The spell or ability that the enemy attacked you with."],
	},
	color = "ffff00", -- yellow
}

Parrot:RegisterFilterType("Incoming heals", L["Incoming heals"], 0)

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Heals",
	localName = L["Heals"],
	defaultTag = "([Skill] - [Name]) +[Amount]",
	parserEvent = {
		eventType = "Heal",
		recipientID = "player",
		isHoT = false,
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = retrieveAbilityName,
		Amount = "realAmount",
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the ally that healed you."],
		Skill = L["The spell or ability that the ally healed you with."],
		Amount = L["The amount of healing done."],
	},
	color = "00ff00", -- green
	canCrit = true,
	filterType = { "Incoming heals", 'realAmount' },
}

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Heals over time",
	localName = L["Heals over time"],
	defaultTag = "([Skill] - [Name]) +[Amount]",
	parserEvent = {
		eventType = "Heal",
		recipientID = "player",
		isHoT = true,
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = retrieveAbilityName,
		Amount = "realAmount",
		Count = "throttleCount",
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the ally that healed you."],
		Skill = L["The spell or ability that the ally healed you with."],
		Amount = L["The amount of healing done."],
		Count = L["The amount of spell casts to make up the throttled event."],
	},
	throttle = { "DoTs and HoTs", 'abilityName', 'throttleCount', sourceName = L["Multiple"] },
	color = "00ff00", -- green
	filterType = { "Incoming heals", 'realAmount' },
}

Parrot:RegisterCombatEvent{
	category = "Incoming",
	name = "Environmental damage",
	localName = L["Environmental damage"],
	defaultTag = "-[Amount] [Type]",
	parserEvent = {
		eventType = "Environmental",
		recipientID = "player",
	},
	tagTranslations = {
		Amount = coloredDamageAmount,
		Type = "hazardTypeLocal",
	},
	tagTranslationsHelp = {
		Amount = L["The amount of damage done."],
		Type = L["The type of damage done."],
	},
	color = "ff0000", -- red
}

------------------------------------------------------------------------------
-- Outgoing events -----------------------------------------------------------
------------------------------------------------------------------------------
Parrot:RegisterFilterType("Outgoing damage", L["Outgoing damage"], 0)

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee damage",
	localName = L["Melee damage"],
	defaultTag = "[Amount]",
	parserEvent = {
		eventType = "Damage",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Amount = coloredDamageAmount,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
		Amount = L["The amount of damage done."],
	},
	color = "ffffff", -- white
	canCrit = true,
	filterType = { "Outgoing damage", 'amount' },
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee misses",
	localName = L["Melee misses"],
	defaultTag = L["Miss!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Miss",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
	},
	color = "ffffff", -- white
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee dodges",
	localName = L["Melee dodges"],
	defaultTag = L["Dodge!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Dodge",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
	},
	color = "ffffff", -- white
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee parries",
	localName = L["Melee parries"],
	defaultTag = L["Parry!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Parry",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
	},
	color = "ffffff", -- white
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee blocks",
	localName = L["Melee blocks"],
	defaultTag = L["Block!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Block",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
	},
	color = "ffffff", -- white
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee absorbs",
	localName = L["Melee absorbs"],
	defaultTag = L["Absorb!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Absorb",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
	},
	color = "ffff00", -- yellow
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee immunes",
	localName = L["Melee immunes"],
	defaultTag = L["Immune!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Immune",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
	},
	color = "ffff00", -- yellow
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee evades",
	localName = L["Melee evades"],
	defaultTag = L["Evade!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Evade",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
	},
	color = "ff7f00", -- orange
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Skill damage",
	localName = L["Skill damage"],
	defaultTag = "[Amount] ([Skill])",
	parserEvent = {
		eventType = "Damage",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
		isDoT = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Amount = coloredDamageAmount,
		Type = "damageTypeLocal",
		Skill = retrieveAbilityName,
		Count = "throttleCount",
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
		Amount = L["The amount of damage done."],
		Type = L["The type of damage done."],
		Skill = L["The spell or ability that you used."],
		Count = L["The amount of spell casts to make up the throttled event."],
	},
	color = "ffff00", -- yellow
	canCrit = true,
	throttle = { "Skill damage", 'abilityName', 'throttleCount', recipientName = L["Multiple"] },
	filterType = { "Outgoing damage", 'amount' },
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Skill DoTs",
	localName = L["Skill DoTs"],
	defaultTag = "[Amount] ([Skill])",
	parserEvent = {
		eventType = "Damage",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
		isDoT = true,
	},
	tagTranslations = {
		Name = "recipientName",
		Amount = coloredDamageAmount,
		Type = "damageTypeLocal",
		Skill = retrieveAbilityName,
		Count = "throttleCount",
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
		Amount = L["The amount of damage done."],
		Type = L["The type of damage done."],
		Skill = L["The spell or ability that you used."],
		Count = L["The amount of spell casts to make up the throttled event."],
	},
	throttle = { "DoTs and HoTs", 'abilityName', 'throttleCount', recipientName = L["Multiple"] },
	color = "ffff00", -- yellow
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Ability misses",
	localName = L["Ability misses"],
	defaultTag = L["Miss!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Miss",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
		Skill = L["The spell or ability that you used."],
	},
	color = "ffffff", -- white
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Ability dodges",
	localName = L["Ability dodges"],
	defaultTag = L["Dodge!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Dodge",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
		Skill = L["The spell or ability that you used."],
	},
	color = "ffffff", -- white
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Ability parries",
	localName = L["Ability parries"],
	defaultTag = L["Parry!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Parry",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
		Skill = L["The spell or ability that you used."],
	},
	color = "ffffff", -- white
}


Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Ability blocks",
	localName = L["Ability blocks"],
	defaultTag = L["Block!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Block",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
		Skill = L["The spell or ability that you used."],
	},
	color = "ffffff", -- white
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Spell resists",
	localName = L["Spell resists"],
	defaultTag = L["Resist!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Resist",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
		Skill = L["The spell or ability that you used."],
	},
	color = "7f7fb2", -- blue-gray
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Skill absorbs",
	localName = L["Skill absorbs"],
	defaultTag = L["Absorb!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Absorb",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
		Skill = L["The spell or ability that you used."],
	},
	color = "ffff00", -- yellow
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Skill immunes",
	localName = L["Skill immunes"],
	defaultTag = L["Immune!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Immune",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
		Skill = L["The spell or ability that you used."],
	},
	color = "ffff00", -- yellow
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Skill reflects",
	localName = L["Skill reflects"],
	defaultTag = L["Reflect!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Reflect",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
		Skill = L["The spell or ability that you used."],
	},
	color = "ffff00", -- yellow
	canCrit = true,
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Spell interrupts",
	localName = L["Skill interrupts"],
	defaultTag = L["Interrupt!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Interrupt",
		sourceID = "player",
		recipientID_not = "player",
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
		Skill = L["The spell or ability that you used."],
	},
	color = "ffff00", -- yellow
}


Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Skill evades",
	localName = L["Skill evades"],
	defaultTag = L["Evade!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Evade",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy you attacked."],
		Skill = L["The spell or ability that you used."],
	},
	color = "ff7f00", -- orange
}

Parrot:RegisterFilterType("Outgoing heals", L["Outgoing heals"], 0)

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Heals",
	localName = L["Heals"],
	defaultTag = "+[Amount] ([Skill] - [Name])",
	parserEvent = {
		eventType = "Heal",
		sourceID = "player",
		recipientID_not = "player",
		isHoT = false,
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Amount = "realAmount",
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the ally you healed."],
		Skill = L["The spell or ability that you used."],
		Amount = L["The amount of healing done."],
	},
	color = "00ff00", -- green
	canCrit = true,
	filterType = { "Outgoing heals", 'realAmount' },
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Heals over time",
	localName = L["Heals over time"],
	defaultTag = "+[Amount] ([Skill] - [Name])",
	parserEvent = {
		eventType = "Heal",
		sourceID = "player",
		recipientID_not = "player",
		isHoT = true,
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Amount = "realAmount",
		Count = "throttleCount",
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the ally you healed."],
		Skill = L["The spell or ability that you used."],
		Amount = L["The amount of healing done."],
		Count = L["The amount of spell casts to make up the throttled event."],
	},
	throttle = { "DoTs and HoTs", 'abilityName', 'throttleCount', recipientName = L["Multiple"] },
	color = "00ff00", -- green
	filterType = { "Outgoing heals", 'realAmount' },
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet melee damage",
	localName = L["Pet melee damage"],
	defaultTag = "Pet [Amount]",
	parserEvent = {
		eventType = "Damage",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Amount = coloredDamageAmount,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
		Amount = L["The amount of damage done."],
	},
	color = "ff7f00", -- orange
	canCrit = true,
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet melee misses",
	localName = L["Pet melee misses"],
	defaultTag = L["Pet Miss!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Miss",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
	},
	color = "ff7f00", -- orange
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet melee dodges",
	localName = L["Pet melee dodges"],
	defaultTag = L["Pet Dodge!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Dodge",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
	},
	color = "ff7f00", -- orange
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet melee parries",
	localName = L["Pet melee parries"],
	defaultTag = L["Pet Parry!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Parry",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
	},
	color = "ff7f00", -- orange
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet melee blocks",
	localName = L["Pet melee blocks"],
	defaultTag = L["Pet Block!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Block",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
	},
	color = "ff7f00", -- orange
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet melee absorbs",
	localName = L["Pet melee absorbs"],
	defaultTag = L["Pet Absorb!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Absorb",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
	},
	color = "7f7fff", -- light blue
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet melee immunes",
	localName = L["Pet melee immunes"],
	defaultTag = L["Pet Immune!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Immune",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
	},
	color = "7f7fff", -- light blue
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet melee evades",
	localName = L["Pet melee evades"],
	defaultTag = L["Pet Evade!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Evade",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
	},
	color = "ff7fff", -- pink
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet skill damage",
	localName = L["Pet skill damage"],
	defaultTag = "Pet [Amount] ([Skill])",
	parserEvent = {
		eventType = "Damage",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Amount = coloredDamageAmount,
		Type = "damageTypeLocal",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
		Amount = L["The amount of damage done."],
		Type = L["The type of damage done."],
		Skill = L["The ability or spell your pet used."],
	},
	color = "0000ff", -- blue
	canCrit = true,
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet ability misses",
	localName = L["Pet ability misses"],
	defaultTag = L["Pet Miss!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Miss",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
		Skill = L["The ability or spell your pet used."],
	},
	color = "0000ff", -- blue
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet ability dodges",
	localName = L["Pet ability dodges"],
	defaultTag = L["Pet Dodge!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Dodge",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
		Skill = L["The ability or spell your pet used."],
	},
	color = "0000ff", -- blue
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet ability parries",
	localName = L["Pet ability parries"],
	defaultTag = L["Pet Parry!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Parry",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
		Skill = L["The ability or spell your pet used."],
	},
	color = "0000ff", -- blue
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet ability blocks",
	localName = L["Pet ability blocks"],
	defaultTag = L["Pet Block!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Block",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
		Skill = L["The ability or spell your pet used."],
	},
	color = "0000ff", -- blue
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet spell resists",
	localName = L["Pet spell resists"],
	defaultTag = L["Pet Resist!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Resist",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
		Skill = L["The ability or spell your pet used."],
	},
	color = "7f7fb2", -- blue-gray
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet skill absorbs",
	localName = L["Pet skill"],
	defaultTag = L["Pet Absorb!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Absorb",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
		Skill = L["The ability or spell your pet used."],
	},
	color = "7f7fff", -- light blue
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet skill immunes",
	localName = L["Pet skill immunes"],
	defaultTag = L["Pet Immune!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Immune",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
		Skill = L["The ability or spell your pet used."],
	},
	color = "7f7fff", -- light blue
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet skill reflects",
	localName = L["Pet skill reflects"],
	defaultTag = L["Pet Reflect!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Reflect",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
		Skill = L["The ability or spell your pet used."],
	},
	color = "7f7fff", -- light blue
	canCrit = true,
}

Parrot:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet skill evades",
	localName = L["Pet skill evades"],
	defaultTag = L["Evade!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Evade",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Name = L["The name of the enemy your pet attacked."],
		Skill = L["The ability or spell your pet used."],
	},
	color = "ff7fff", -- pink
}

------------------------------------------------------------------------------
-- Notification events -------------------------------------------------------
------------------------------------------------------------------------------

Parrot:RegisterCombatEvent{
	category = "Notification",
	name = "Enter combat",
	localName = L["Enter combat"],
	defaultTag = "+Combat",
	blizzardEvent = "PLAYER_REGEN_DISABLED",
	color = "ffffff", -- white
}

Parrot:RegisterCombatEvent{
	category = "Notification",
	name = "Leave combat",
	localName = L["Leave combat"],
	defaultTag = "-Combat",
	blizzardEvent = "PLAYER_REGEN_ENABLED",
	color = "ffffff", -- white
}

Parrot:RegisterThrottleType("Power gain/loss", L["Power gain/loss"], 3)
Parrot:RegisterFilterType("Power gain", L["Power gain"], 0)

Parrot:RegisterCombatEvent{
	category = "Notification",
	name = "Power gain",
	localName = L["Power gain"],
	defaultTag = L["+[Amount] [Type]"],
	parserEvent = {
		eventType = "Gain",
		attribute_in = { "Mana", "Focus", "Rage", "Energy" },
		recipientID = "player",
	},
	tagTranslations = {
		Amount = function(info)
			return info.amountGained or info.amount
		end,
		Type = function(info)
			return info.attributeGainedLocal or info.attributeLocal
		end,
		Skill = retrieveAbilityName,
		Name = function(info)
			return info.sourceGainedName and info.recipientName or info.sourceName
		end,
		Count = "throttleCount",
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Amount = L["The amount of power gained."],
		Type = L["The type of power gained (Mana, Rage, Energy)."],
		Skill = L["The ability or spell used to gain power."],
		Name = L["The character that the power comes from."],
		Count = L["The amount of spell casts to make up the throttled event."],
	},
	color = "ffff00", -- yellow
	throttle = { "Power gain/loss", 'abilityName', 'throttleCount', sourceName = L["Multiple"], recipientName = L["Multiple"] },
	filterType = { "Power gain", function(info)
		return info.amountGained or info.amount
	end}
}

onEnableFuncs[#onEnableFuncs+1] = function()
	mod:RegisterParserEvent({
		eventType = "Leech",
		sourceGainedID = "player",
		attributeGained_in = { "Mana", "Focus", "Rage", "Energy" },
	}, function(info)
		Parrot:TriggerCombatEvent("Notification", "Power gain", info)
	end)
end

Parrot:RegisterCombatEvent{
	category = "Notification",
	name = "Power loss",
	localName = L["Power loss"],
	defaultTag = L["-[Amount] [Type]"],
	parserEvent = {
		eventType_in = { "Drain", "Leech" },
		attribute_in = { "Mana", "Focus", "Rage", "Energy" },
		recipientID = "player",
	},
	tagTranslations = {
		Amount = "amount",
		Type = "attributeLocal",
		Skill = retrieveAbilityName,
		Name = "sourceName",
		Count = "throttleCount",
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationsHelp = {
		Amount = L["The amount of power lost."],
		Type = L["The type of power lost (Mana, Rage, Energy)."],
		Skill = L["The ability or spell take away your power."],
		Name = L["The character that caused the power loss."],
		Count = L["The amount of spell casts to make up the throttled event."],
	},
	color = "ffff00", -- yellow
	throttle = { "Power gain/loss", 'abilityName', 'throttleCount', sourceName = L["Multiple"] },
}

Parrot:RegisterCombatEvent{
	category = "Notification",
	name = "Combo point gain",
	localName = L["Combo point gain"],
	defaultTag = L["[Num] CP"],
	tagTranslations = {
		Num = 1
	},
	tagTranslationHelp = {
		Num = L["The current number of combo points."]
	},
	color = "ff7f00", -- orange
}

Parrot:RegisterCombatEvent{
	category = "Notification",
	name = "Combo points full",
	localName = L["Combo points full"],
	defaultTag = L["[Num] CP Finish It!"],
	tagTranslations = {
		Num = 1
	},
	tagTranslationHelp = {
		Num = L["The current number of combo points."]
	},
	color = "ff7f00", -- orange
}

onEnableFuncs[#onEnableFuncs+1] = function()
	mod:RegisterEvent("PLAYER_COMBO_POINTS", function()
		local num = GetComboPoints()
		if num == 0 then
			return
		end
		local info = new(num)
		Parrot:TriggerCombatEvent("Notification", num == 5 and "Combo points full" or "Combo point gain", info)
		info = del(info)
	end)
end

Parrot:RegisterCombatEvent{
	category = "Notification",
	name = "Honor gains",
	localName = L["Honor gains"],
	defaultTag = "+[Amount] " .. HONOR_CONTRIBUTION_POINTS,
	parserEvent = {
		eventType = "Honor",
		amount_gt = 0,
		isHonorable = true,
	},
	tagTranslations = {
		Amount = "amount",
		Name = "sourceName",
		Rank = "sourceRank",
	},
	tagTranslationHelp = {
		Amount = L["The amount of honor gained."],
		Name = L["The name of the enemy slain."],
		Rank = L["The rank of the enemy slain."],
	},
	color = "7f7fb2", -- blue-gray
}

Parrot:RegisterCombatEvent{
	category = "Notification",
	name = "Reputation gains",
	localName = L["Reputation gains"],
	defaultTag = "+[Amount] " .. REPUTATION .. " ([Faction])",
	parserEvent = {
		eventType = "Reputation",
		amount_gt = 0,
	},
	tagTranslations = {
		Amount = "amount",
		Faction = "faction",
	},
	tagTranslationHelp = {
		Amount = L["The amount of reputation gained."],
		Faction = L["The name of the faction."],
	},
	color = "7f7fb2", -- blue-gray
}

Parrot:RegisterCombatEvent{
	category = "Notification",
	name = "Reputation losses",
	localName = L["Reputation losses"],
	defaultTag = "-[Amount] " .. REPUTATION .. " ([Faction])",
	parserEvent = {
		eventType = "Reputation",
		amount_lt = 0,
	},
	tagTranslations = {
		Amount = function(info) return -info.amount end,
		Faction = "faction",
	},
	tagTranslationHelp = {
		Amount = L["The amount of reputation lost."],
		Faction = L["The name of the faction."],
	},
	color = "7f7fb2", -- blue-gray
}

Parrot:RegisterCombatEvent{
	category = "Notification",
	name = "Skill gains",
	localName = L["Skill gains"],
	defaultTag = "[Skill]: [Amount]",
	parserEvent = {
		eventType = "Skill",
	},
	tagTranslations = {
		Skill = retrieveAbilityName,
		Amount = "amount",
	},
	tagTranslationHelp = {
		Skill = L["The skill which experienced a gain."],
		Amount = L["The amount of skill points currently."]
	},
	color = "5555ff", -- semi-light blue
}

Parrot:RegisterCombatEvent{
	category = "Notification",
	name = "Experience gains",
	localName = L["Experience gains"],
	defaultTag = "[Amount] " .. XP,
	parserEvent = {
		eventType = "Experience",
		recipientID = "player",
	},
	tagTranslations = {
		Name = "sourceName",
		Amount = "amount",
	},
	tagTranslationHelp = {
		Name = L["The name of the enemy slain."],
		Amount = L["The amount of experience points gained."]
	},
	color = "bf4ccc", -- magenta
	sticky = true,
	defaultDisabled = true,
}

Parrot:RegisterCombatEvent{
	category = "Notification",
	name = "Player killing blows",
	localName = L["Player killing blows"],
	defaultTag = L["Killing Blow!"] .. " ([Name])",
	parserEvent = {
		eventType = "Death",
		sourceID = "player",
		recipientID_not = "player",
		recipientPvP = true,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = function(info) return info.abilityName or PLAYERSTAT_MELEE_COMBAT end,
	},
	tagTranslationHelp = {
		Name = L["The name of the enemy slain."],
		Skill = L["The spell or ability used to slay the enemy."],
	},
	color = "5555ff", -- semi-light blue
	sticky = true,
}

Parrot:RegisterCombatEvent{
	category = "Notification",
	name = "NPC killing blows",
	localName = L["NPC killing blows"],
	defaultTag = L["Killing Blow!"] .. " ([Name])",
	parserEvent = {
		eventType = "Death",
		sourceID = "player",
		recipientID_not = "player",
		recipientPvP = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = function(info) return info.abilityName or PLAYERSTAT_MELEE_COMBAT end,
	},
	tagTranslationHelp = {
		Name = L["The name of the enemy slain."],
		Skill = L["The spell or ability used to slay the enemy."],
	},
	color = "5555ff", -- semi-light blue
	sticky = true,
}

if playerClass == "WARLOCK" then
	Parrot:RegisterCombatEvent{
		category = "Notification",
		name = "Soul shard gains",
		localName = L["Soul shard gains"],
		defaultTag = "+[Name]",
		parserEvent = {
			eventType = "Create",
			itemString_match = "item:6265:[%-%d:]+",
			isCreated = true,
		},
		tagTranslations = {
			Name = "itemName",
			Icon = function(info)
				local itemLink = info.itemLink
				if itemLink then
					local _, _, _, _, _, _, _, _, _, texture = GetItemInfo(itemLink)
					return texture
				end
			end
		},
		tagTranslationHelp = {
			Name = L["The name of the soul shard."],
		},
		color = "990099", -- purple
		sticky = true,
	}
end

Parrot:RegisterCombatEvent{
	category = "Notification",
	name = "Extra attacks",
	localName = L["Extra attacks"],
	defaultTag = L["%s!"]:format("[Skill]"),
	parserEvent = {
		eventType = "Extra Attack",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Skill = retrieveAbilityName,
		Icon = retrieveIconFromAbilityName,
	},
	tagTranslationHelp = {
		Skill = L["The name of the spell or ability which provided the extra attacks."],
	},
	color = "ffff00", -- yellow
	sticky = true,
}
